@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Outc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Outc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -382,13 +382,13 @@ zip_flush (GsfOutfileZip *zip)
 
 	do {
 		zret = deflate (zip->stream, Z_FINISH);
-		if (zret == Z_OK) {
-			/*  In this case Z_OK means more buffer space
-			    needed  */
+		if (zret == Z_OK || (zret == Z_BUF_ERROR && zip->stream->avail_out == 0)) {
+			/*  In this case Z_OK or Z_BUF_ERROR means more buffer 
+			    space is needed  */
 			if (!zip_output_block (zip))
 				return FALSE;
 		}
-	} while (zret == Z_OK);
+	} while (zret == Z_OK || zret == Z_BUF_ERROR);
 	if (zret != Z_STREAM_END)
 		return FALSE;
 	if (!zip_output_block (zip))
@@ -739,9 +739,10 @@ gsf_outfile_zip_new (GsfOutput *sink, G_GNUC_UNUSED GError **err)
 					   NULL);
 }
 
+/* deprecated has no effect */
 gboolean
-gsf_outfile_zip_set_compression_method (GsfOutfileZip *zip,
-					GsfZipCompressionMethod method)
+gsf_outfile_zip_set_compression_method (G_GNUC_UNUSED GsfOutfileZip *zip,
+					G_GNUC_UNUSED GsfZipCompressionMethod method)
 {
 	return TRUE;
 }
