@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -36,12 +36,11 @@ clone (GsfInfile *in, GsfOutfile *out)
 {
 	GsfInput *input = GSF_INPUT (in);
 	GsfOutput *output = GSF_OUTPUT (out);
-	guint8 const *data;
-	size_t len;
-	int i;
 
 	if (gsf_input_size (input) > 0) {
+		size_t len;
 		while ((len = gsf_input_remaining (input)) > 0) {
+			guint8 const *data;
 			/* copy in odd sized chunks to exercise system */
 			if (len > 314)
 				len = 314;
@@ -54,12 +53,35 @@ clone (GsfInfile *in, GsfOutfile *out)
 				return;
 			}
 		}
-	} else for (i = 0 ; i < gsf_infile_num_children (in) ; i++) {
-		input = gsf_infile_child_by_index (in, i);
-		output = gsf_outfile_new_child  (out,
-			gsf_infile_name_by_index  (in, i),
-			gsf_infile_num_children (GSF_INFILE (input)) >= 0);
-		clone (GSF_INFILE (input), GSF_OUTFILE (output));
+	} else {
+		int i, n = gsf_infile_num_children (in);
+		for (i = 0 ; i < n; i++) {
+			const char *name;
+			char *display_name;
+			int level;
+			gboolean is_dir;
+
+			input = gsf_infile_child_by_index (in, i);
+			name = gsf_infile_name_by_index (in, i);
+			is_dir = gsf_infile_num_children (GSF_INFILE (input)) >= 0;
+
+			g_object_get (G_OBJECT (input), "compression-level", &level, NULL);
+
+			display_name = name
+				? g_filename_display_name (name)
+				: NULL;
+			g_print ("%s: size=%ld, level=%d, %s\n",
+				 display_name ? display_name : "?",
+				 (long)gsf_input_size (input),
+				 level,
+				 is_dir ? "directory" : "file");
+			g_free (display_name);
+
+			output = gsf_outfile_new_child_full  (out, name, is_dir,
+							      "compression-level", level,
+							      NULL);
+			clone (GSF_INFILE (input), GSF_OUTFILE (output));
+		}
 	}
 	gsf_output_close (GSF_OUTPUT (out));
 	g_object_unref (G_OBJECT (out));
@@ -76,7 +98,7 @@ test (char *argv[])
 	GError    *err;
 
 	fprintf (stderr, "%s\n", argv [1]);
-	input = GSF_INPUT (gsf_input_stdio_new (argv[1], &err));
+	input = gsf_input_stdio_new (argv[1], &err);
 	if (input == NULL) {
 
 		g_return_val_if_fail (err != NULL, 1);
@@ -86,7 +108,7 @@ test (char *argv[])
 		return 1;
 	}
 
-	infile = GSF_INFILE (gsf_infile_zip_new (input, &err));
+	infile = gsf_infile_zip_new (input, &err);
 	g_object_unref (G_OBJECT (input));
 
 	if (infile == NULL) {
@@ -97,7 +119,7 @@ test (char *argv[])
 		return 1;
 	}
 
-	output = GSF_OUTPUT (gsf_output_stdio_new (argv[2], &err));
+	output = gsf_output_stdio_new (argv[2], &err);
 	if (output == NULL) {
 
 		g_return_val_if_fail (err != NULL, 1);
@@ -108,7 +130,7 @@ test (char *argv[])
 		return 1;
 	}
 
-	outfile = GSF_OUTFILE (gsf_outfile_zip_new (output, &err));
+	outfile = gsf_outfile_zip_new (output, &err);
 	g_object_unref (G_OBJECT (output));
 	clone (infile, outfile);
 
